=== renamed file 'database/schema/pending/fix-teamparticipation.py' => 'cronscripts/check-teamparticipation.py'
--- database/schema/pending/fix-teamparticipation.py	2007-09-13 20:49:21 +0000
+++ cronscripts/check-teamparticipation.py	2007-09-14 15:49:37 +0000
@@ -1,45 +1,55 @@
 #!/usr/bin/python2.4
 # Copyright 2007 Canonical Ltd.  All rights reserved.
 
+"""Check for invalid TeamParticipation entries.
+
+These are the ones for which there are no active TeamMemberships leading to.
+
+This script is usually run on staging to find discrepancies between the
+TeamMembership and TeamParticipation tables wich are a good indication of
+bugs in the code which maintains the TeamParticipation table.
+
+Ideally there should be database constraints to prevent this sort of
+situation, but that's not a simple thing and this should do for now.
+"""
+
 import _pythonpath
 
 from canonical.database.sqlbase import cursor
-from canonical.launchpad.database import Person, TeamParticipation
+from canonical.launchpad.database import Person
 from canonical.lp import initZopeless
 
 
-ztm = initZopeless(implicitBegin=False)
-
-ztm.begin()
-query = """
-    SELECT DISTINCT Person.id
-    FROM Person, TeamParticipation
-    WHERE Person.id = Teamparticipation.person
-        AND TeamParticipation.team != Person.id
-    """
-cur = cursor()
-cur.execute(query)
-people_ids = cur.fetchall()
-ztm.abort()
-
-batch = people_ids[:50]
-people_ids = people_ids[50:]
-removed_entries = 0
-while batch:
-    for [id] in batch:
-        ztm.begin()
-        person = Person.get(id)
-        for team in person.teams_indirectly_participated_in:
-            try:
-                path = person.findPathToTeam(team)
-            except AssertionError, e:
-                print ("Removing TeamParticipation entry for %s on %s"
-                       % (person.unique_displayname, team.unique_displayname))
-                tp = TeamParticipation.selectOneBy(person=person, team=team)
-                tp.destroySelf()
-                removed_entries += 1
-        ztm.commit()
+if __name__ == '__main__':
+    ztm = initZopeless(implicitBegin=False)
+
+    ztm.begin()
+    query = """
+        SELECT DISTINCT Person.id
+        FROM Person, TeamParticipation
+        WHERE Person.id = Teamparticipation.person
+            AND TeamParticipation.team != Person.id
+        """
+    cur = cursor()
+    cur.execute(query)
+    people_ids = cur.fetchall()
+    ztm.abort()
+
     batch = people_ids[:50]
     people_ids = people_ids[50:]
+    while batch:
+        for [id] in batch:
+            ztm.begin()
+            person = Person.get(id)
+            for team in person.teams_indirectly_participated_in:
+                try:
+                    path = person.findPathToTeam(team)
+                except AssertionError, e:
+                    print ("Invalid teamParticipation entry for %s (%d) on %s "
+                           "(%d) -- there's no team membership leading to that"
+                           % (person.unique_displayname, person.id,
+                              team.unique_displayname, team.id))
+            ztm.abort()
+        batch = people_ids[:50]
+        people_ids = people_ids[50:]
 
-print "Removed %d entries in total." % removed_entries

=== modified file 'lib/canonical/launchpad/ftests/test_teammembership.py'
--- lib/canonical/launchpad/ftests/test_teammembership.py	2007-09-14 13:11:59 +0000
+++ lib/canonical/launchpad/ftests/test_teammembership.py	2007-09-14 15:49:37 +0000
@@ -2,6 +2,7 @@
 
 __metaclass__ = type
 
+import subprocess
 import unittest
 from datetime import datetime
 
@@ -11,7 +12,7 @@
 
 from canonical.database.sqlbase import (
     flush_database_caches, flush_database_updates, cursor)
-from canonical.launchpad.database import TeamMembership
+from canonical.launchpad.database import TeamMembership, TeamParticipation
 from canonical.launchpad.ftests import login
 from canonical.launchpad.interfaces import (
     IPersonSet, ITeamMembershipSet, TeamMembershipStatus)
@@ -113,6 +114,45 @@
         self.assertEqual(new_status, TeamMembershipStatus.DEACTIVATED.value)
 
 
+class TestCheckTeamParticipationScript(unittest.TestCase):
+    layer = LaunchpadFunctionalLayer
+
+    def _runScript(self):
+        process = subprocess.Popen(
+            'cronscripts/check-teamparticipation.py', shell=True,
+            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE)
+        (out, err) = process.communicate()
+        self.assertEqual(process.returncode, 0, (out, err))
+        return out, err
+
+    def test_no_output_if_no_invalid_entries(self):
+        """No output if there's no invalid teamparticipation entries."""
+        out, err = self._runScript()
+        self.assertEqual((out, err), ('', ''))
+
+    def test_report_invalid_teamparticipation_entries(self):
+        cur = cursor()
+        cur.execute("""
+            SELECT p.id, t.id
+            FROM person AS p, person AS t
+            WHERE (p.id, t.id) NOT IN (
+                SELECT person, team FROM teamparticipation)
+                AND p.id != t.id
+                AND p.teamowner IS NULL
+                AND t.teamowner IS NOT NULL
+                LIMIT 1
+            """)
+        [person, team] = cur.fetchone()
+        tp = TeamParticipation(person=person, team=team)
+        import transaction
+        transaction.commit()
+        out, err = self._runScript()
+        self.assertEqual(err, '', (out, err))
+        self.failUnless(
+            'Invalid teamParticipation entry for' in out, (out, err))
+
+
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 

